#
# LECA Bash library
#
# The LECA bash library provides a set of function used for helping
# development of bash script mainly to write job script on the luke
# cluster
#
# The files from the LECA Bash library must be sourced from your main script
#

# For inheritance of the ERR trapping to ON
# Which is not the default behaviour
set -o errtrace

include logging
logdebug "Load trap package"


include stacks

__LECABASH__trap_signal_list__() {
	echo $(kill -l \
             | sed -E $'s/[ \t]+/@/g' \
             | tr '@' '\n' \
             | grep '^SIG' \
             | sed 's/SIG//') EXIT ERR
}

__LECABASH__trap_signal_code__() {
	local signal=$1
	echo $(kill -l              \
	        | sed -E $'s/[ \t]+/@/g' \
	        | sed -E 's/\)@/ /g'     \
	        | tr '@' '\n'            \
	        | grep "SIG${signal}"    \
	        | awk '{print $1}') \
    || echo 0
}


__LECABASH__signal_trap_stack_name__() {
  local signal="$1"
  local stackname="__LECABASH__signal_${signal}_trap_stack__"
			
  echo $stackname
}

__LECABASH__trap_function__() {
	local exitcode="$?"
	local signal="$1"
	local stackname="$(__LECABASH__signal_trap_stack_name__ ${signal})"
	local code="$(__LECABASH__trap_signal_code__ $signal)"
	
	logdebug "Signal ${signal} trapped"
		
	if [[ $(stacksize $stackname) > 0 ]]; then
		while popvalue "$stackname" cmd ; do
			logdebug "Running ${signal} command : $cmd"
			eval "$cmd" \
			   || logdebug "Command: '$cmd' generates an error" 
		done
		logdebug "Commands registered on signal $signal run."
	fi
	
	if [[ "$signal" != "EXIT" ]]; then
		exit "$((128 + code))"
	else
		exit "$exitcode"
    fi
}

__LECABASH__contains__() {
  local e match="$1"
  shift
  
  for e in "$@"; do 
  	[[ "$e" == "$match" ]] && return 0
  done
  
  return 1
}

#
# Check if the signal is already traped by the LECABashLib
#
__LECABASH__istrapregistred__() {
	local signal="$1"
	local registred=$(trap -p "$signal" \
	                    | grep '__LECABASH__trap_function__' \
	                    | wc -l)
	                    
	if (( registred == 0 )) ; then
		return 1
    else
		return 0
    fi
}

function __LECABASH__registered_trap__() {
	local signal="$1"
    
    trap -p "$signal"                     \
         | awk '{$NF="";                  \
                 split($0,parts,"--");    \
                 print parts[2]}'         \
         | sed -E 's/^[ '\'']+//'           \
         | sed -E 's/'\'' +$//'           

}

function __LECABASH__kill_running_command__() {
  local signal=$1
  local pid=$2

  loginfo "Sending signal ${signal} to process ${pid}"
  kill -s ${signal} ${pid}
  
}


function registertrap() {
	local signal=$1
	shift
	local cmd="${*}"
	local stackname="$(__LECABASH__signal_trap_stack_name__ ${signal})"
	
	if ! __LECABASH__istrapregistred__ "$signal" ; then
		newstack $stackname

		local oldcmd=$(__LECABASH__registered_trap__ "$signal")
		if [[ oldcmd != "" ]] ; then
			logdebug "Save previous trap function for signal $signal" 
			pushvalue "$stackname" "${oldcmd}"
	    fi 
	    
	    logdebug "Register trap for signal $signal"
		trap "__LECABASH__trap_function__ $signal" "$signal"
    fi
	
	logdebug "Register new command on $signal : ${cmd}"
	pushvalue "$stackname" "${cmd}"
}


function run() {
  local signal=$1
  shift
  local signalto=$1
  shift

  local stackname="$(__LECABASH__signal_trap_stack_name__ ${signal})"

  eval ${*} &
  fpid=$!
  registertrap ${signal} __LECABASH__kill_running_command__ ${signalto} ${fpid}
  killid=$(( $(stacksize "$stackname") - 1 ))
  
  while [ -e /proc/$fpid ]
	do
  		sleep 1
	done
	
  logdebug "Remove kill action on $fpid for $signal signal" 
  eval "${stackname}[${killid}]='sleep 0'"
}

function signalSpy() {
	for s in $(__LECABASH__trap_signal_list__) ; do
		registertrap $s loginfo Signal $s has been emited
	done
}